@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2009-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: utils.h 11490 2010-12-08 14:57:34Z charles $
+ * $Id: utils.h 12211 2011-03-22 23:49:29Z jordan $
  */
 
 #ifndef TR_UTILS_H
@@ -15,9 +15,6 @@
 
 #include <inttypes.h>
 #include <stddef.h> /* size_t */
-#include <stdio.h> /* FILE* */
-#include <string.h> /* memcpy()* */
-#include <stdlib.h> /* malloc() */
 #include <time.h> /* time_t */
 
 #ifdef __cplusplus
@@ -33,14 +30,6 @@ extern "C" {
  * @{
  */
 
-#ifndef FALSE
- #define FALSE 0
-#endif
-
-#ifndef TRUE
- #define TRUE 1
-#endif
-
 #ifndef UNUSED
  #ifdef __GNUC__
   #define UNUSED __attribute__ ( ( unused ) )
@@ -76,10 +65,8 @@ extern "C" {
 #endif
 
 #if __GNUC__ > 2 || ( __GNUC__ == 2 && __GNUC_MINOR__ >= 96 )
- #define TR_GNUC_PURE __attribute__ ( ( __pure__ ) )
  #define TR_GNUC_MALLOC __attribute__ ( ( __malloc__ ) )
 #else
- #define TR_GNUC_PURE
  #define TR_GNUC_MALLOC
 #endif
 
@@ -101,7 +88,7 @@ const char * tr_strip_positional_args( const char * fmt );
 
 /* #define DISABLE_GETTEXT */
 #ifndef DISABLE_GETTEXT
- #if defined(WIN32) || defined(TR_EMBEDDED)
+ #if defined(WIN32) || defined(TR_LIGHTWEIGHT)
    #define DISABLE_GETTEXT
  #endif
 #endif
@@ -116,11 +103,16 @@ const char * tr_strip_positional_args( const char * fmt );
 
 #define TR_MAX_MSG_LOG 10000
 
-extern tr_msg_level messageLevel;
+extern tr_msg_level __tr_message_level;
 
-static inline tr_bool tr_msgLoggingIsActive( tr_msg_level level )
+static inline tr_msg_level tr_getMessageLevel( void )
 {
-    return messageLevel >= level;
+    return __tr_message_level;
+}
+
+static inline bool tr_msgLoggingIsActive( tr_msg_level level )
+{
+    return tr_getMessageLevel() >= level;
 }
 
 void tr_msg( const char * file, int line,
@@ -184,10 +176,10 @@ void tr_msg( const char * file, int line,
 
 
 
-FILE*          tr_getLog( void );
+void* tr_getLog( void );
 
 /** @brief return true if deep logging has been enabled by the user; false otherwise */
-tr_bool tr_deepLoggingIsActive( void );
+bool tr_deepLoggingIsActive( void );
 
 void           tr_deepLog( const char * file,
                            int          line,
@@ -203,7 +195,7 @@ char* tr_getLogTimeStr( char * buf, int buflen ) TR_GNUC_NONNULL(1);
  * @brief Rich Salz's classic implementation of shell-style pattern matching for ?, \, [], and * characters.
  * @return 1 if the pattern matches, 0 if it doesn't, or -1 if an error occured
  */
-int tr_wildmat( const char * text, const char * pattern ) TR_GNUC_NONNULL(1,2);
+bool tr_wildmat( const char * text, const char * pattern ) TR_GNUC_NONNULL(1,2);
 
 /** @brief Portability wrapper for basename() that uses the system implementation if available */
 char* tr_basename( const char * path ) TR_GNUC_MALLOC;
@@ -272,7 +264,7 @@ void tr_wait_msec( long int delay_milliseconds );
  * @brief make a copy of 'str' whose non-utf8 content has been corrected or stripped
  * @return a newly-allocated string that must be freed with tr_free()
  * @param str the string to make a clean copy of
- * @param len the length of the string to copy.  If -1, the entire string is used.
+ * @param len the length of the string to copy. If -1, the entire string is used.
  */
 char* tr_utf8clean( const char * str, int len ) TR_GNUC_MALLOC;
 
@@ -312,20 +304,20 @@ void tr_free( void * p );
 void* tr_memdup( const void * src, size_t byteCount );
 
 #define tr_new( struct_type, n_structs )           \
-    ( (struct_type *) tr_malloc ( ( (size_t) sizeof ( struct_type ) ) * ( ( size_t) ( n_structs ) ) ) )
+    ( (struct_type *) tr_malloc ( sizeof ( struct_type ) * ( ( size_t) ( n_structs ) ) ) )
 
 #define tr_new0( struct_type, n_structs )          \
-    ( (struct_type *) tr_malloc0 ( ( (size_t) sizeof ( struct_type ) ) * ( ( size_t) ( n_structs ) ) ) )
+    ( (struct_type *) tr_malloc0 ( sizeof ( struct_type ) * ( ( size_t) ( n_structs ) ) ) )
 
 #define tr_renew( struct_type, mem, n_structs )    \
-    ( (struct_type *) realloc ( ( mem ), ( (size_t) sizeof ( struct_type ) ) * ( ( size_t) ( n_structs ) ) ) )
+    ( (struct_type *) realloc ( ( mem ), sizeof ( struct_type ) * ( ( size_t) ( n_structs ) ) ) )
 
 void* tr_valloc( size_t bufLen );
 
 /**
  * @brief make a newly-allocated copy of a substring
  * @param in is a void* so that callers can pass in both signed & unsigned without a cast
- * @param len length of the substring to copy.  if a length less than zero is passed in, strlen( len ) is used
+ * @param len length of the substring to copy. if a length less than zero is passed in, strlen( len ) is used
  * @return a newly-allocated copy of `in' that can be freed with tr_free()
  */
 char* tr_strndup( const void * in, int len ) TR_GNUC_MALLOC;
@@ -337,13 +329,24 @@ char* tr_strndup( const void * in, int len ) TR_GNUC_MALLOC;
  */
 char* tr_strdup( const void * in );
 
+/**
+ * @brief like strcmp() but gracefully handles NULL strings
+ */
+int tr_strcmp0( const char * str1, const char * str2 );
+
+
+
+struct evbuffer;
+
+char* evbuffer_free_to_str( struct evbuffer * buf );
+
 /** @brief similar to bsearch() but returns the index of the lower bound */
 int tr_lowerBound( const void * key,
                    const void * base,
                    size_t       nmemb,
                    size_t       size,
                    int       (* compar)(const void* key, const void* arrayMember),
-                   tr_bool    * exact_match ) TR_GNUC_HOT TR_GNUC_NONNULL(1,5,6);
+                   bool       * exact_match ) TR_GNUC_HOT TR_GNUC_NONNULL(1,5,6);
 
 
 /**
@@ -385,7 +388,7 @@ const char* tr_strerror( int );
 char* tr_strstrip( char * str );
 
 /** @brief Returns true if the string ends with the specified case-insensitive suffix */
-tr_bool tr_str_has_suffix( const char *str, const char *suffix );
+bool tr_str_has_suffix( const char *str, const char *suffix );
 
 
 /** @brief Portability wrapper for memmem() that uses the system implementation if available */
@@ -399,30 +402,6 @@ char* tr_strsep( char ** str, const char * delim );
 ****
 ***/
 
-typedef void ( tr_set_func )( void * element, void * userData );
-
-/**
- * @brief find the differences and commonalities in two sorted sets
- * @param a the first set
- * @param aCount the number of elements in the set 'a'
- * @param b the second set
- * @param bCount the number of elements in the set 'b'
- * @param compare the sorting method for both sets
- * @param elementSize the sizeof the element in the two sorted sets
- * @param in_a called for items in set 'a' but not set 'b'
- * @param in_b called for items in set 'b' but not set 'a'
- * @param in_both called for items that are in both sets
- * @param userData user data passed along to in_a, in_b, and in_both
- */
-void tr_set_compare( const void * a, size_t aCount,
-                     const void * b, size_t bCount,
-                     int compare( const void * a, const void * b ),
-                     size_t elementSize,
-                     tr_set_func in_a_cb,
-                     tr_set_func in_b_cb,
-                     tr_set_func in_both_cb,
-                     void * userData );
-
 int compareInt( const void * va, const void * vb );
 
 void tr_sha1_to_hex( char * out, const uint8_t * sha1 ) TR_GNUC_NONNULL(1,2);
@@ -430,13 +409,13 @@ void tr_sha1_to_hex( char * out, const uint8_t * sha1 ) TR_GNUC_NONNULL(1,2);
 void tr_hex_to_sha1( uint8_t * out, const char * hex ) TR_GNUC_NONNULL(1,2);
 
 /** @brief convenience function to determine if an address is an IP address (IPv4 or IPv6) */
-tr_bool tr_addressIsIP( const char * address );
+bool tr_addressIsIP( const char * address );
 
-/** @brief return TRUE if the url is a http or https url that Transmission understands */
-tr_bool tr_urlIsValidTracker( const char * url ) TR_GNUC_NONNULL(1);
+/** @brief return true if the url is a http or https url that Transmission understands */
+bool tr_urlIsValidTracker( const char * url ) TR_GNUC_NONNULL(1);
 
-/** @brief return TRUE if the url is a [ http, https, ftp, ftps ] url that Transmission understands */
-tr_bool tr_urlIsValid( const char * url, int url_len ) TR_GNUC_NONNULL(1);
+/** @brief return true if the url is a [ http, https, ftp, ftps ] url that Transmission understands */
+bool tr_urlIsValid( const char * url, int url_len ) TR_GNUC_NONNULL(1);
 
 /** @brief parse a URL into its component parts
     @return zero on success or an error number if an error occurred */
@@ -492,9 +471,6 @@ char* tr_strpercent( char * buf, double x, size_t buflen );
  */
 char* tr_strratio( char * buf, size_t buflen, double ratio, const char * infinity ) TR_GNUC_NONNULL(1,4);
 
-/* return a truncated double as a string */
-char* tr_strtruncd( char * buf, double x, int precision, size_t buflen );
-
 /** @brief Portability wrapper for localtime_r() that uses the system implementation if available */
 struct tm * tr_localtime_r( const time_t *_clock, struct tm *_result );
 
@@ -504,7 +480,10 @@ struct tm * tr_localtime_r( const time_t *_clock, struct tm *_result );
  * @return 0 on success; otherwise, return -1 and set errno
  */
 int tr_moveFile( const char * oldpath, const char * newpath,
-                 tr_bool * renamed ) TR_GNUC_NONNULL(1,2);
+                 bool * renamed ) TR_GNUC_NONNULL(1,2);
+
+/** @brief Test to see if the two filenames point to the same file. */
+bool tr_is_same_file( const char * filename1, const char * filename2 );
 
 /** @brief convenience function to remove an item from an array */
 void tr_removeElementFromArray( void         * array,
@@ -517,7 +496,7 @@ void tr_removeElementFromArray( void         * array,
 ***/
 
 /** @brief Private libtransmission variable that's visible only for inlining in tr_time() */
-extern time_t transmission_now;
+extern time_t __tr_current_time;
 
 /**
  * @brief very inexpensive form of time(NULL)
@@ -526,17 +505,23 @@ extern time_t transmission_now;
  * This function returns a second counter that is updated once per second.
  * If something blocks the libtransmission thread for more than a second,
  * that counter may be thrown off, so this function is not guaranteed
- * to always be accurate.  However, it is *much* faster when 100% accuracy
+ * to always be accurate. However, it is *much* faster when 100% accuracy
  * isn't needed
  */
-static inline time_t tr_time( void ) { return transmission_now; }
+static inline time_t tr_time( void ) { return __tr_current_time; }
 
 /** @brief Private libtransmission function to update tr_time()'s counter */
-static inline void tr_timeUpdate( time_t now ) { transmission_now = now; }
+static inline void tr_timeUpdate( time_t now ) { __tr_current_time = now; }
 
 /** @brief Portability wrapper for realpath() that uses the system implementation if available */
 char* tr_realpath( const char *path, char * resolved_path );
 
+/** @brief Portability wrapper for htonll() that uses the system implementation if available */
+uint64_t tr_htonll( uint64_t );
+
+/** @brief Portability wrapper for htonll() that uses the system implementation if available */
+uint64_t tr_ntohll( uint64_t );
+
 /***
 ****
 ***/
@@ -560,13 +545,13 @@ extern unsigned int tr_size_K;
 char* tr_formatter_speed_KBps( char * buf, double KBps, size_t buflen );
 
 /* format a memory size from bytes into a user-readable string. */
-char* tr_formatter_mem_B( char * buf, uint64_t bytes, size_t buflen );
+char* tr_formatter_mem_B( char * buf, int64_t bytes, size_t buflen );
 
 /* format a memory size from MB into a user-readable string. */
 static inline char* tr_formatter_mem_MB( char * buf, double MBps, size_t buflen ) { return tr_formatter_mem_B( buf, MBps * tr_mem_K * tr_mem_K, buflen ); }
 
 /* format a file size from bytes into a user-readable string. */
-char* tr_formatter_size_B( char * buf, uint64_t bytes, size_t buflen );
+char* tr_formatter_size_B( char * buf, int64_t bytes, size_t buflen );
 
 void tr_formatter_get_units( struct tr_benc * dict );
 
