@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: TrackerCell.m 10527 2010-04-24 19:39:25Z livings124 $
+ * $Id: TrackerCell.m 12370 2011-04-17 20:01:46Z livings124 $
  * 
- * Copyright (c) 2009-2010 Transmission authors and contributors
+ * Copyright (c) 2009-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -196,32 +196,36 @@ NSMutableSet * fTrackerIconLoading;
 
 - (NSImage *) favIcon
 {
+    id icon = nil;
     NSURL * address = [NSURL URLWithString: [(TrackerNode *)[self objectValue] fullAnnounceAddress]];
-    NSString * host = [address host];
-    
-    //don't try to parse ip address
-    const BOOL separable = !tr_addressIsIP([host UTF8String]);
-    
-    NSArray * hostComponents = separable ? [host componentsSeparatedByString: @"."] : nil;
-    
-    //let's try getting the tracker address without using any subdomains
-    NSString * baseAddress;
-    if (separable && [hostComponents count] > 1)
-        baseAddress = [NSString stringWithFormat: @"http://%@.%@",
-                        [hostComponents objectAtIndex: [hostComponents count]-2], [hostComponents lastObject]];
-    else
-        baseAddress = [NSString stringWithFormat: @"http://%@", host];
-    
-    id icon = [fTrackerIconCache objectForKey: baseAddress];
-    if (!icon && ![fTrackerIconLoading containsObject: baseAddress])
+    NSString * host;
+    if ((host = [address host]))
     {
-        [fTrackerIconLoading addObject: baseAddress];
-        [NSThread detachNewThreadSelector: @selector(loadTrackerIcon:) toTarget: self withObject: baseAddress];
+        //don't try to parse ip address
+        const BOOL separable = !tr_addressIsIP([host UTF8String]);
+        
+        NSArray * hostComponents = separable ? [host componentsSeparatedByString: @"."] : nil;
+        
+        //let's try getting the tracker address without using any subdomains
+        NSString * baseAddress;
+        if (separable && [hostComponents count] > 1)
+            baseAddress = [NSString stringWithFormat: @"http://%@.%@",
+                            [hostComponents objectAtIndex: [hostComponents count]-2], [hostComponents lastObject]];
+        else
+            baseAddress = [NSString stringWithFormat: @"http://%@", host];
+        
+        icon = [fTrackerIconCache objectForKey: baseAddress];
+        if (!icon && ![fTrackerIconLoading containsObject: baseAddress])
+        {
+            [fTrackerIconLoading addObject: baseAddress];
+            [NSThread detachNewThreadSelector: @selector(loadTrackerIcon:) toTarget: self withObject: baseAddress];
+        }
     }
-    
+        
     return (icon && icon != [NSNull null]) ? icon : [NSImage imageNamed: @"FavIcon.png"];
 }
 
+#warning better favicon detection
 - (void) loadTrackerIcon: (NSString *) baseAddress
 {
     NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
