@@ -1,23 +1,28 @@
 /*
- * This file Copyright (C) 2009-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
- * Transmission project are granted a special exemption to clause 2(b)
- * so that the bulk of its code can remain under the MIT license.
- * This exemption does not extend to derived works not owned by
- * the Transmission project.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2
+ * as published by the Free Software Foundation.
  *
- * $Id: filters.cc 9868 2010-01-04 21:00:47Z charles $
+ * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+ *
+ * $Id: filters.cc 11196 2010-09-06 00:19:37Z charles $
  */
 
 #include "filters.h"
 
-const QString FilterMode::names[NUM_MODES] = {
+const QString FilterMode::names[NUM_MODES] =
+{
     "show-all",
     "show-active",
     "show-downloading",
     "show-seeding",
-    "show-paused"
+    "show-paused",
+    "show-finished",
+    "show-queued",
+    "show-verifying",
+    "show-error",
 };
 
 int
@@ -38,7 +43,6 @@ const QString SortMode::names[NUM_MODES] = {
     "sort-by-ratio",
     "sort-by-size",
     "sort-by-state",
-    "sort-by-tracker",
     "sort-by-id"
 };
 
